@@ -441,12 +441,34 @@ class _InvariantTranspiler(
         if isinstance(node.antecedent, no_parentheses_types_in_this_context):
             not_antecedent = f"!{antecedent}"
         else:
-            not_antecedent = f"!({antecedent})"
+            # NOTE (mristin, 2022-04-07):
+            # This is a very rudimentary heuristic for breaking the lines, and can be
+            # greatly improved by rendering into C# code. However, at this point, we
+            # lack time for more sophisticated reformatting approaches.
+            if "\n" in antecedent:
+                not_antecedent = f"""\
+!(
+{I}{indent_but_first_line(antecedent, I)}
+)"""
+            else:
+                not_antecedent = f"!({antecedent})"
 
         if not isinstance(node.consequent, no_parentheses_types_in_this_context):
-            consequent = Stripped(f"({consequent})")
+            # NOTE (mristin, 2022-04-07):
+            # This is a very rudimentary heuristic for breaking the lines, and can be
+            # greatly improved by rendering into C# code. However, at this point, we
+            # lack time for more sophisticated reformatting approaches.
+            if "\n" in consequent:
+                consequent = Stripped(
+                    f"""\
+(
+{I}{indent_but_first_line(consequent, I)}
+)"""
+                )
+            else:
+                consequent = Stripped(f"({consequent})")
 
-        return Stripped(f"{not_antecedent}\n" f"|| {consequent}"), None
+        return Stripped(f"{not_antecedent}\n|| {consequent}"), None
 
     @ensure(lambda result: (result[0] is not None) ^ (result[1] is not None))
     def transform_method_call(
@@ -631,7 +653,7 @@ class _InvariantTranspiler(
         elif isinstance(node.value, (int, float)):
             return Stripped(str(node.value)), None
         elif isinstance(node.value, str):
-            return Stripped(repr(node.value)), None
+            return Stripped(csharp_common.string_literal(node.value)), None
         else:
             assert_never(node.value)
 
@@ -741,7 +763,19 @@ class _InvariantTranspiler(
             )
 
             if not isinstance(value_node, no_parentheses_types_in_this_context):
-                value = Stripped(f"({value})")
+                # NOTE (mristin, 2022-04-07):
+                # This is a very rudimentary heuristic for breaking the lines, and can
+                # be greatly improved by rendering into C# code. However, at this point,
+                # we lack time for more sophisticated reformatting approaches.
+                if "\n" in value:
+                    value = Stripped(
+                        f"""\
+(
+{I}{indent_but_first_line(value, I)}
+)"""
+                    )
+                else:
+                    value = Stripped(f"({value})")
 
             values.append(value)
 
@@ -750,9 +784,14 @@ class _InvariantTranspiler(
                 node.original_node, "Failed to transpile the conjunction", errors
             )
 
-        # BEFORE-RELEASE (mristin, 2021-12-13):
-        #  add heuristic for breaking the lines
-        return Stripped(" && ".join(values)), None
+        writer = io.StringIO()
+        for i, value in enumerate(values):
+            if i == 0:
+                writer.write(value)
+            else:
+                writer.write(f"\n&& {value}")
+
+        return Stripped(writer.getvalue()), None
 
     def transform_or(
         self, node: parse_tree.Or
@@ -777,7 +816,19 @@ class _InvariantTranspiler(
             )
 
             if not isinstance(value_node, no_parentheses_types_in_this_context):
-                value = Stripped(f"({value})")
+                # NOTE (mristin, 2022-04-07):
+                # This is a very rudimentary heuristic for breaking the lines, and can
+                # be greatly improved by rendering into C# code. However, at this point,
+                # we lack time for more sophisticated reformatting approaches.
+                if "\n" in value:
+                    value = Stripped(
+                        f"""\
+(
+{I}{indent_but_first_line(value, I)}
+)"""
+                    )
+                else:
+                    value = Stripped(f"({value})")
 
             values.append(value)
 
@@ -786,9 +837,14 @@ class _InvariantTranspiler(
                 node.original_node, "Failed to transpile the conjunction", errors
             )
 
-        # BEFORE-RELEASE (mristin, 2021-12-13):
-        #  add heuristic for breaking the lines
-        return Stripped(" || ".join(values)), None
+        writer = io.StringIO()
+        for i, value in enumerate(values):
+            if i == 0:
+                writer.write(value)
+            else:
+                writer.write(f"\n|| {value}")
+
+        return Stripped(writer.getvalue()), None
 
     def transform_joined_str(
         self, node: parse_tree.JoinedStr
@@ -870,13 +926,14 @@ class _InvariantTranspiler(
         ):
             iteration = Stripped(f"({iteration})")
 
+        # NOTE (mristin, 2022-04-07):
+        # We can not use ``textwrap.dedent`` here since ``condition`` contains multiple
+        # lines.
         return (
             Stripped(
-                textwrap.dedent(
-                    f"""\
-                {iteration}.All(
-                {I}{variable} => {condition})"""
-                )
+                f"""\
+{iteration}.All(
+{I}{variable} => {indent_but_first_line(condition, II)})"""
             ),
             None,
         )
@@ -888,6 +945,88 @@ assert all(
 )
 
 
+@ensure(lambda text, result: text == "".join(result))
+def _wrap_invariant_description(text: str) -> List[str]:
+    """
+    Wrap the invariant description as ``text`` into multiple tokens.
+
+    The tokens are split based on the whitespace. We make sure the articles are not
+    left hanging between the lines. A line should observe a pre-defined line limit,
+    if possible.
+
+    No new lines are added — the description should be given to the user in the original
+    formatting. We merely split it in string literals for better code readability.
+    """
+    parts = text.split(" ")
+    if len(parts) == 1:
+        return [text]
+
+    # NOTE (mristin, 2022-04-08):
+    # We do not want to cut out "the", "a" and "an" on separate lines, so we split
+    # the text once more in tokens where the articles are kept in the same token as
+    # the word.
+    tokens = []  # type: List[str]
+
+    article = None  # type: Optional[str]
+    for part in parts:
+        if article is None:
+            if part in ("a", "an", "the"):
+                article = part
+                continue
+            else:
+                tokens.append(part)
+        else:
+            if part in ("a", "an", "the"):
+                # Append the previously observed ``article``;
+                # the ``part`` becomes a new article.
+                tokens.append(article)
+                article = part
+                continue
+
+            tokens.append(f"{article} {part}")
+            article = None
+
+    if article is not None:
+        tokens.append(article)
+
+    # NOTE (mristin, 2022-04-08):
+    # We add space to the tokens so that it is easier to re-flow them.
+    tokens = [
+        f"{token} " if i < len(tokens) - 1 else token for i, token in enumerate(tokens)
+    ]
+    assert "".join(tokens) == text
+
+    # NOTE (mristin, 2022-04-08):
+    # The line width of 60 characters is an arbitrary, but plausible limit. Please
+    # consider that the text will be indented, so you have to add some slack.
+    line_width = 60
+
+    segments = []  # type: List[str]
+
+    accumulation_len = 0
+    accumulation = []  # type: List[str]
+
+    for token in tokens:
+        if len(token) > line_width:
+            segments.append("".join(accumulation))
+            segments.append(token)
+            accumulation_len = 0
+            accumulation = []
+
+        elif accumulation_len + len(token) > line_width:
+            segments.append("".join(accumulation))
+            accumulation_len = len(token)
+            accumulation = [token]
+        else:
+            accumulation_len += len(token)
+            accumulation.append(token)
+
+    if accumulation_len > 0:
+        segments.append("".join(accumulation))
+
+    return segments
+
+
 @ensure(lambda result: (result[0] is not None) ^ (result[1] is not None))
 def _transpile_invariant(
     invariant: intermediate.Invariant,
@@ -956,18 +1095,32 @@ def _transpile_invariant(
         )
     )
 
-    lines = []  # type: List[str]
+    message_literals = []  # type: List[Stripped]
     if invariant.description is not None:
-        lines = invariant.description.splitlines()
+        # NOTE (mristin, 2022-04-08):
+        # We need to wrap the description in multiple literals as a single long
+        # string literal is often too much for the readability.
+        invariant_description_lines = _wrap_invariant_description(invariant.description)
+        for i, line in enumerate(invariant_description_lines):
+            if i < len(invariant_description_lines) - 1:
+                message_literals.append(csharp_common.string_literal(line))
+            else:
+                message_literals.append(csharp_common.string_literal(f"{line}\n"))
 
-    lines = lines + expr.splitlines()
-
-    for i, line in enumerate(lines):
-        if i < len(lines) - 1:
-            line_literal = csharp_common.string_literal(line + "\n")
-            writer.write(f"{II}{line_literal} +\n")
+    expr_lines = expr.splitlines()
+    for i, line in enumerate(expr_lines):
+        if i < len(expr_lines) - 1:
+            literal = csharp_common.string_literal(line + "\n")
         else:
-            writer.write(f"{II}{csharp_common.string_literal(line)});")
+            literal = csharp_common.string_literal(line)
+
+        message_literals.append(literal)
+
+    for i, literal in enumerate(message_literals):
+        if i < len(message_literals) - 1:
+            writer.write(f"{II}{literal} +\n")
+        else:
+            writer.write(f"{II}{literal});")
 
     writer.write("\n}")
 
