@@ -16,7 +16,6 @@ from aas_core_codegen.intermediate._types import (
     Default,
     DefaultConstant,
     DefaultEnumerationLiteral,
-    Description,
     Enumeration,
     EnumerationLiteral,
     ImplementationSpecificMethod,
@@ -31,16 +30,19 @@ from aas_core_codegen.intermediate._types import (
     PatternVerification,
     PrimitiveTypeAnnotation,
     Property,
-    RefTypeAnnotation,
     Serialization,
     Signature,
-    SignatureLike,
     Snapshot,
     Symbol,
     SymbolTable,
-    TypeAnnotation,
-    Verification,
     UnderstoodMethod,
+    ReferenceInTheBook,
+    MetaModelDescription,
+    SymbolDescription,
+    SignatureDescription,
+    PropertyDescription,
+    EnumerationLiteralDescription,
+    SignatureLike,
 )
 from aas_core_codegen.parse import tree as parse_tree
 
@@ -101,13 +103,21 @@ def _stringify_optional_type_annotation(
     return result
 
 
-def _stringify_ref_type_annotation(
-    that: RefTypeAnnotation,
+def _stringify_meta_model_description(
+    that: MetaModelDescription,
 ) -> stringify.Entity:
     result = stringify.Entity(
         name=that.__class__.__name__,
         properties=[
-            stringify.Property("value", _stringify(that.value)),
+            stringify.Property("summary", str(that.summary)),
+            stringify.Property("remarks", list(map(str, that.remarks))),
+            stringify.Property(
+                "constraints_by_identifier",
+                [
+                    [identifier, str(body)]
+                    for identifier, body in that.constraints_by_identifier.items()
+                ],
+            ),
             stringify.PropertyEllipsis("parsed", that.parsed),
         ],
     )
@@ -115,14 +125,81 @@ def _stringify_ref_type_annotation(
     return result
 
 
-def _stringify_description(
-    that: Description,
+def _stringify_symbol_description(
+    that: SymbolDescription,
 ) -> stringify.Entity:
     result = stringify.Entity(
         name=that.__class__.__name__,
         properties=[
-            stringify.PropertyEllipsis("document", that.document),
-            stringify.PropertyEllipsis("node", that.node),
+            stringify.Property("summary", str(that.summary)),
+            stringify.Property("remarks", list(map(str, that.remarks))),
+            stringify.Property(
+                "constraints_by_identifier",
+                [
+                    [identifier, str(body)]
+                    for identifier, body in that.constraints_by_identifier.items()
+                ],
+            ),
+            stringify.PropertyEllipsis("parsed", that.parsed),
+        ],
+    )
+
+    return result
+
+
+def _stringify_property_description(
+    that: PropertyDescription,
+) -> stringify.Entity:
+    result = stringify.Entity(
+        name=that.__class__.__name__,
+        properties=[
+            stringify.Property("summary", str(that.summary)),
+            stringify.Property("remarks", list(map(str, that.remarks))),
+            stringify.Property(
+                "constraints_by_identifier",
+                [
+                    [identifier, str(body)]
+                    for identifier, body in that.constraints_by_identifier.items()
+                ],
+            ),
+            stringify.PropertyEllipsis("parsed", that.parsed),
+        ],
+    )
+
+    return result
+
+
+def _stringify_enumeration_literal_description(
+    that: EnumerationLiteralDescription,
+) -> stringify.Entity:
+    result = stringify.Entity(
+        name=that.__class__.__name__,
+        properties=[
+            stringify.Property("summary", str(that.summary)),
+            stringify.Property("remarks", list(map(str, that.remarks))),
+            stringify.PropertyEllipsis("parsed", that.parsed),
+        ],
+    )
+
+    return result
+
+
+def _stringify_signature_description(
+    that: SignatureDescription,
+) -> stringify.Entity:
+    result = stringify.Entity(
+        name=that.__class__.__name__,
+        properties=[
+            stringify.Property("summary", str(that.summary)),
+            stringify.Property("remarks", list(map(str, that.remarks))),
+            stringify.Property(
+                "arguments_by_name",
+                [[name, str(body)] for name, body in that.arguments_by_name.items()],
+            ),
+            stringify.Property(
+                "returns", None if that.returns is None else str(that.returns)
+            ),
+            stringify.PropertyEllipsis("parsed", that.parsed),
         ],
     )
 
@@ -215,6 +292,21 @@ def _stringify_serialization(
     return result
 
 
+def _stringify_reference_in_the_book(
+    that: ReferenceInTheBook,
+) -> stringify.Entity:
+    result = stringify.Entity(
+        name=that.__class__.__name__,
+        properties=[
+            stringify.Property("section", list(that.section)),
+            stringify.Property("index", that.index),
+            stringify.Property("fragment", that.fragment),
+        ],
+    )
+
+    return result
+
+
 def _stringify_invariant(
     that: Invariant,
 ) -> stringify.Entity:
@@ -402,6 +494,9 @@ def _stringify_enumeration(
                     for parent_enum in that.is_superset_of
                 ],
             ),
+            stringify.Property(
+                "reference_in_the_book", _stringify(that.reference_in_the_book)
+            ),
             stringify.Property("description", _stringify(that.description)),
             stringify.PropertyEllipsis("literals_by_name", that.literals_by_name),
             stringify.PropertyEllipsis("literal_id_set", that.literal_id_set),
@@ -434,6 +529,9 @@ def _stringify_constrained_primitive(
             ),
             stringify.Property("invariants", list(map(_stringify, that.invariants))),
             stringify.PropertyEllipsis("invariant_id_set", that.invariant_id_set),
+            stringify.Property(
+                "reference_in_the_book", _stringify(that.reference_in_the_book)
+            ),
             stringify.Property("description", _stringify(that.description)),
             stringify.PropertyEllipsis("parsed", that.parsed),
         ],
@@ -476,6 +574,9 @@ def _stringify_a_class(that: Class) -> stringify.Entity:
             stringify.Property("constructor", _stringify(that.constructor)),
             stringify.Property("invariants", list(map(_stringify, that.invariants))),
             stringify.Property("serialization", _stringify(that.serialization)),
+            stringify.Property(
+                "reference_in_the_book", _stringify(that.reference_in_the_book)
+            ),
             stringify.Property("description", _stringify(that.description)),
             stringify.PropertyEllipsis("parsed", that.parsed),
             stringify.PropertyEllipsis("properties_by_name", that.properties_by_name),
@@ -581,6 +682,13 @@ def _stringify_symbol_table(
         name=that.__class__.__name__,
         properties=[
             stringify.Property("symbols", list(map(_stringify, that.symbols))),
+            stringify.Property(
+                "symbols_topologically_sorted",
+                [
+                    f"Reference to symbol {symbol.name}"
+                    for symbol in that.symbols_topologically_sorted
+                ],
+            ),
             stringify.Property(
                 "verification_functions",
                 list(map(_stringify, that.verification_functions)),
@@ -588,11 +696,6 @@ def _stringify_symbol_table(
             stringify.PropertyEllipsis(
                 "verification_functions_by_name", that.verification_functions_by_name
             ),
-            stringify.Property(
-                "ref_association",
-                f"Reference to {that.ref_association.__class__.__name__} "
-                f"{that.ref_association.name}",
-            ),
             stringify.Property("meta_model", _stringify(that.meta_model)),
         ],
     )
@@ -603,13 +706,16 @@ def _stringify_symbol_table(
 Dumpable = Union[
     AbstractClass,
     Argument,
-    Class,
     ConcreteClass,
     Constructor,
     Contract,
     Contracts,
     Default,
-    Description,
+    MetaModelDescription,
+    SymbolDescription,
+    PropertyDescription,
+    EnumerationLiteralDescription,
+    SignatureDescription,
     Enumeration,
     EnumerationLiteral,
     ImplementationSpecificMethod,
@@ -618,21 +724,17 @@ Dumpable = Union[
     Invariant,
     ListTypeAnnotation,
     MetaModel,
-    Method,
     OptionalTypeAnnotation,
     OurTypeAnnotation,
     PatternVerification,
     PrimitiveTypeAnnotation,
     Property,
-    RefTypeAnnotation,
+    ReferenceInTheBook,
     Serialization,
     Signature,
-    SignatureLike,
     Snapshot,
     Symbol,
     SymbolTable,
-    TypeAnnotation,
-    Verification,
     UnderstoodMethod,
 ]
 
@@ -650,7 +752,11 @@ _DISPATCH = {
     Contracts: _stringify_contracts,
     DefaultConstant: _stringify_default_constant,
     DefaultEnumerationLiteral: _stringify_default_enumeration_literal,
-    Description: _stringify_description,
+    MetaModelDescription: _stringify_meta_model_description,
+    SymbolDescription: _stringify_symbol_description,
+    PropertyDescription: _stringify_property_description,
+    EnumerationLiteralDescription: _stringify_enumeration_literal_description,
+    SignatureDescription: _stringify_signature_description,
     Enumeration: _stringify_enumeration,
     EnumerationLiteral: _stringify_enumeration_literal,
     ImplementationSpecificMethod: _stringify_implementation_specific_method,
@@ -664,7 +770,7 @@ _DISPATCH = {
     PatternVerification: _stringify_pattern_verification,
     PrimitiveTypeAnnotation: _stringify_primitive_type_annotation,
     Property: _stringify_property,
-    RefTypeAnnotation: _stringify_ref_type_annotation,
+    ReferenceInTheBook: _stringify_reference_in_the_book,
     Serialization: _stringify_serialization,
     Signature: _stringify_signature,
     Snapshot: _stringify_snapshot,
