@@ -624,6 +624,28 @@ class Snapshot:
 class Contracts:
     """Represent the set of contracts for a method or a function."""
 
+    # NOTE (mristin, 2022-04-07):
+    # Common programming languages which work with contracts usually implement
+    # pre-conditions in a disjunctive normal form, *i.e.* as a disjunction of
+    # conjunctions, where at least one conjunction needs to hold. The individual
+    # conjunctions correspond to the levels of the inheritance hierarchy.
+    #
+    # However, we have not touched methods at the moment nor their proper inheritance.
+    # Therefore we leave the pre-conditions in the intermediate representation as they
+    # would appear in the code, without inheritance and hence without disjunctions.
+    # In the future, once we want to tackle the methods as a feature, we need to change
+    # the way how we model and resolve the pre-conditions through
+    # the inheritance hierarchy.
+
+    #: Pre-conditions that need to hold *before* the call
+    preconditions: Final[Sequence[Contract]]
+
+    #: Snapshots which are captured *before* the call
+    snapshots: Final[Sequence[Snapshot]]
+
+    #: Post-conditions that need to hold *after* the call
+    postconditions: Final[Sequence[Contract]]
+
     def __init__(
         self,
         preconditions: Sequence[Contract],
@@ -1125,6 +1147,7 @@ class ConstrainedPrimitive:
         self.description = description
         self.parsed = parsed
 
+    @require(lambda self, descendants: self not in descendants)
     def _set_descendants(self, descendants: Sequence["ConstrainedPrimitive"]) -> None:
         """
         Set the descendants in the constrained primitive.
@@ -1372,6 +1395,7 @@ class Class(DBC):
             id(inheritance) for inheritance in self._inheritances
         )
 
+    @require(lambda self, descendants: self not in descendants)
     def _set_descendants(self, descendants: Sequence["ClassUnion"]) -> None:
         """
         Set the descendants and the concrete descendants in the class.
